@@ -2,21 +2,22 @@ import datetime as dt
 import operator
 import random
 
-from keg.db import db
-from sqlalchemy_utils import ArrowType, EmailType
 import arrow
 import blazeutils.strings
 import pytz
 import six
 import sqlalchemy as sa
 import wrapt
+from blazeutils import tolist
+from keg.db import db
+from sqlalchemy.ext.hybrid import hybrid_property
+from sqlalchemy_utils import ArrowType, EmailType
 
-import keg_elements.decorators as decor
 import keg_elements.db.columns as columns
 import keg_elements.db.utils as dbutils
+import keg_elements.decorators as decor
 from keg_elements.extensions import lazy_gettext as _
 
-
 might_commit = decor.keyword_optional('_commit', after=dbutils.session_commit, when_missing=True)
 might_flush = decor.keyword_optional('_flush', after=dbutils.session_flush)
 
@@ -280,6 +281,53 @@ class MethodsMixin:
             return blazeutils.strings.randchars(min(column.type.length or 25, 25))
         raise ValueError(_('No randomization for this column type'))
 
+    @classmethod
+    def testing_set_related(cls, kwargs, model,
+                            *testing_create_args, **testing_create_kwargs):
+        """Create a related object for testing, if it is not specified in kwargs.
+
+        Designed to be used by `testing_create`. A common issue is that related
+        test records need to be set up with the test entity instance, but they
+        could be specified already in kwargs. In addition, relationships already
+        specified may be given on the relationship attribute or the foreign-key
+        field.
+
+        This method takes existing `testing_create` kwargs and the related
+        entity, and makes the necessary updates.
+
+        Relationship name is generated from the given entity by default, but may
+        be passed in as a `_relationship_name` keyword argument. The foreign key
+        field is assumed to be the relationship name with an `_id` suffix, but
+        may be specified with the `_relationship_field` keyword argument.
+
+        Any additional args/kwargs are passed to the given model's `testing_create`.
+        """
+        relationship_name = testing_create_kwargs.pop(
+            '_relationship_name',
+            blazeutils.strings.case_cw2us(model.__name__),
+        )
+        relationship_field = testing_create_kwargs.pop(
+            '_relationship_field',
+            '{}_id'.format(relationship_name),
+        )
+
+        for attr_name in (relationship_name, relationship_field):
+            if not hasattr(cls, attr_name):
+                print(cls, attr_name)
+                raise Exception(
+                    'testing_set_related expects "{}" on class {}'.format(
+                        attr_name,
+                        cls.__name__,
+                    )
+                )
+
+        # generic logic to respect existing kwargs setting on either field
+        if not {relationship_name, relationship_field} & set(kwargs.keys()):
+            kwargs[relationship_name] = model.testing_create(
+                *testing_create_args,
+                **testing_create_kwargs
+            )
+
     @might_commit
     @might_flush
     @classmethod
@@ -389,3 +437,66 @@ class SoftDeleteMixin:
 
 sa.event.listen(SoftDeleteMixin, 'before_delete', SoftDeleteMixin.sqla_before_delete_event,
                 propagate=True)
+
+
+class LookupMixin(SoftDeleteMixin):
+    """Provides a base for id/label pair tables, used in one-to-many relationships.
+
+    Based on SoftDeleteMixin, so any lookup record that is deleted/deactivated is
+    still available for existing records.
+
+    A code field is provided for developer reference in code, so a changeable label
+    does not need to be hard-coded for lookup.
+
+    Developer expectations:
+    - LookupMixin will precede DefaultMixin or MethodsMixin in entity base classes
+    - Only active labels will be listed for linking to new related records
+    - `include_ids` will be used for ensuring existing records preserve lookup
+    """
+
+    label = sa.Column(sa.Unicode(255), nullable=False, unique=True)
+    code = sa.Column(sa.Unicode(255))
+
+    @hybrid_property
+    def is_active(self):
+        return self.deleted_utc is not None
+
+    @is_active.expression
+    def is_active(cls):
+        return sa.sql.case([(cls.deleted_utc.is_(None), sa.true())], else_=sa.false())
+
+    @classmethod
+    def _active_query(cls, include_ids=None, order_by=None):
+        if order_by is None:
+            order_by = cls.label
+
+        if include_ids:
+            include_ids = tolist(include_ids)
+            clause = sa.sql.or_(
+                cls.is_active == sa.true(),
+                cls.id.in_(include_ids)
+            )
+        else:
+            clause = cls.is_active == sa.true()
+
+        return cls.query.filter(clause).order_by(order_by)
+
+    @classmethod
+    def list_active(cls, include_ids=None, order_by=None):
+        return cls._active_query(include_ids, order_by).all()
+
+    @classmethod
+    def pairs_active(cls, include_ids=None, order_by=None):
+        query = cls._active_query(include_ids, order_by)
+        return cls.pairs('id', 'label', query=query)
+
+    @classmethod
+    def get_by_label(cls, label):
+        return cls.get_by(label=label)
+
+    @classmethod
+    def get_by_code(cls, code):
+        return cls.get_by(code=code)
+
+    def __repr__(self):
+        return '<{} {}:{}>'.format(self.__class__.__name__, self.id, self.label)
